//
// Module: Multiply Add (MADD)
//
// Description: Verilog instantiation template for 
// DSP48 embedded MAC blocks arranged as a Systolic
// 23 tap FIR filter. 23 slices used, 1 slice per tap.
// 
//
// Device: Whitney Family
//
// Copyright (c) 2000 Xilinx, Inc.  All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////////
//


module MADD (CLK, RST, CE, DIN, DOUT);
	
input            CLK, CE, RST;
input   [17:0]   DIN;
output  [47:0]   DOUT;

wire    [47:0]   PCOUT_0, PCOUT_1, PCOUT_2, PCOUT_3, PCOUT_4, PCOUT_5, PCOUT_6, PCOUT_7, PCOUT_8, PCOUT_9, PCOUT_10, PCOUT_11, PCOUT_12, PCOUT_13, PCOUT_14, PCOUT_15, PCOUT_16, PCOUT_17, PCOUT_18, PCOUT_19, PCOUT_20, PCOUT_21;
wire    [17:0]   H0, H1, H2, H3, H4, H5, H6, H7, H8, H9, H10, H11, H12, H13, H14, H15, H16, H17, H18, H19, H20, H21, H22;
wire    [17:0]   BCOUT_0, BCOUT_1, BCOUT_2, BCOUT_3, BCOUT_4, BCOUT_5, BCOUT_6, BCOUT_7, BCOUT_8, BCOUT_9, BCOUT_10, BCOUT_11, BCOUT_12, BCOUT_13, BCOUT_14, BCOUT_15, BCOUT_16, BCOUT_17, BCOUT_18, BCOUT_19, BCOUT_20, BCOUT_21;

/////////////////////////////////////////////////
// COEFFICENT VALUES
////////////////////////////////////////////////

assign H0 = -38;
assign H1 = -74;
assign H2 = -109;
assign H3 = -109;
assign H4 = -37;
assign H5 = 140;
assign H6 = 435;
assign H7 = 827;
assign H8 = 1262;
assign H9 = 1663;
assign H10 = 1945;
assign H11 = 2047;
assign H12 = 1945;
assign H13 = 1663;
assign H14 = 1262;
assign H15 = 827;
assign H16 = 435;
assign H17 = 140;
assign H18 = -37;
assign H19 = -109;
assign H20 = -109;
assign H21 = -74;
assign H22 = -38;
											  
//          
// Tap 0 Instantiation block 1
//

DSP48 MADD0
  (
    .A(H0),             // Input A to Multiplier
    .B(DIN),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(18'b0),          //
    .PCIN(48'b0),          //
    .OPMODE(7'b0000101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
	 .CECINSUB(CE),       //
    .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(BCOUT_0),       //
    .P(),                  //
    .PCOUT(PCOUT_0)        //
  );

//synthesis attribute AREG of MADD0 is "1";
//synthesis attribute BREG of MADD0 is "2";
//synthesis attribute CARRYINREG of MADD0 is "0";
//synthesis attribute MREG of MADD0 is "1";
//synthesis attribute PREG of MADD0 is "1";
//synthesis attribute OPMODEREG of MADD0 is "0";
//synthesis attribute SUBTRACTREG of MADD0 is "0";
//synthesis attribute CARRYINSELREG of MADD0 is "0";
//synthesis attribute B_INPUT of MADD0 is "DIRECT";

//synthesis translate_off 
defparam MADD0.AREG = 2'b01;
defparam MADD0.BREG = 2'b10;
defparam MADD0.CARRYINREG = 1'b0;
defparam MADD0.MREG = 1'b1;
defparam MADD0.PREG = 1'b1;
defparam MADD0.OPMODEREG = 1'b0;
defparam MADD0.SUBTRACTREG = 1'b0;
defparam MADD0.CARRYINSELREG = 1'b0;
defparam MADD0.B_INPUT = "DIRECT";
//synthesis translate_on

//          
// Tap 1 Instantiation block 2
//

DSP48 MADD1
  (
    .A(H1),             // Input A to Multiplier
    .B(18'b0),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(BCOUT_0),          //
    .PCIN(PCOUT_0),          //
    .OPMODE(7'b0010101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
	 .CECINSUB(CE),       //
    .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(BCOUT_1),       //
    .P(),                  //
    .PCOUT(PCOUT_1)        //
  );

//synthesis attribute AREG of MADD1 is "1";
//synthesis attribute BREG of MADD1 is "2";
//synthesis attribute CARRYINREG of MADD1 is "0";
//synthesis attribute MREG of MADD1 is "1";
//synthesis attribute PREG of MADD1 is "1";
//synthesis attribute OPMODEREG of MADD1 is "0";
//synthesis attribute SUBTRACTREG of MADD1 is "0";
//synthesis attribute CARRYINSELREG of MADD1 is "0";
//synthesis attribute B_INPUT of MADD1 is "CASCADE";

//synthesis translate_off 
defparam MADD1.AREG = 2'b01;
defparam MADD1.BREG = 2'b10;
defparam MADD1.CARRYINREG = 1'b0;
defparam MADD1.MREG = 1'b1;
defparam MADD1.PREG = 1'b1;
defparam MADD1.OPMODEREG = 1'b0;
defparam MADD1.SUBTRACTREG = 1'b0;
defparam MADD1.CARRYINSELREG = 1'b0;
defparam MADD1.B_INPUT = "CASCADE";
//synthesis translate_on

//          
// Tap 2 Instantiation block 3
//

DSP48 MADD2
  (
    .A(H2),             // Input A to Multiplier
    .B(18'b0),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(BCOUT_1),          //
    .PCIN(PCOUT_1),          //
    .OPMODE(7'b0010101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
	 .CECINSUB(CE),       //
    .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(BCOUT_2),       //
    .P(),                  //
    .PCOUT(PCOUT_2)        //
  );

//synthesis attribute AREG of MADD2 is "1";
//synthesis attribute BREG of MADD2 is "2";
//synthesis attribute CARRYINREG of MADD2 is "0";
//synthesis attribute MREG of MADD2 is "1";
//synthesis attribute PREG of MADD2 is "1";
//synthesis attribute OPMODEREG of MADD2 is "0";
//synthesis attribute SUBTRACTREG of MADD2 is "0";
//synthesis attribute CARRYINSELREG of MADD2 is "0";
//synthesis attribute B_INPUT of MADD2 is "CASCADE";

//synthesis translate_off 
defparam MADD2.AREG = 2'b01;
defparam MADD2.BREG = 2'b10;
defparam MADD2.CARRYINREG = 1'b0;
defparam MADD2.MREG = 1'b1;
defparam MADD2.PREG = 1'b1;
defparam MADD2.OPMODEREG = 1'b0;
defparam MADD2.SUBTRACTREG = 1'b0;
defparam MADD2.CARRYINSELREG = 1'b0;
defparam MADD2.B_INPUT = "CASCADE";
//synthesis translate_on


//          
// Tap 3 Instantiation block 4
//

DSP48 MADD3
  (
    .A(H3),             // Input A to Multiplier
    .B(18'b0),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(BCOUT_2),          //
    .PCIN(PCOUT_2),          //
    .OPMODE(7'b0010101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
	 .CECINSUB(CE),       //
    .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(BCOUT_3),       //
    .P(),                  //
    .PCOUT(PCOUT_3)        //
  );

//synthesis attribute AREG of MADD3 is "1";
//synthesis attribute BREG of MADD3 is "2";
//synthesis attribute CARRYINREG of MADD3 is "0";
//synthesis attribute MREG of MADD3 is "1";
//synthesis attribute PREG of MADD3 is "1";
//synthesis attribute OPMODEREG of MADD3 is "0";
//synthesis attribute SUBTRACTREG of MADD3 is "0";
//synthesis attribute CARRYINSELREG of MADD3 is "0";
//synthesis attribute B_INPUT of MADD3 is "CASCADE";

//synthesis translate_off 
defparam MADD3.AREG = 2'b01;
defparam MADD3.BREG = 2'b10;
defparam MADD3.CARRYINREG = 1'b0;
defparam MADD3.MREG = 1'b1;
defparam MADD3.PREG = 1'b1;
defparam MADD3.OPMODEREG = 1'b0;
defparam MADD3.SUBTRACTREG = 1'b0;
defparam MADD3.CARRYINSELREG = 1'b0;
defparam MADD3.B_INPUT = "CASCADE";
//synthesis translate_on


//          
// Tap 4 Instantiation block 5
//

DSP48 MADD4
  (
    .A(H4),             // Input A to Multiplier
    .B(18'b0),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(BCOUT_3),          //
    .PCIN(PCOUT_3),          //
    .OPMODE(7'b0010101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
	 .CECINSUB(CE),       //
    .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(BCOUT_4),       //
    .P(),                  //
    .PCOUT(PCOUT_4)        //
  );

//synthesis attribute AREG of MADD4 is "1";
//synthesis attribute BREG of MADD4 is "2";
//synthesis attribute CARRYINREG of MADD4 is "0";
//synthesis attribute MREG of MADD4 is "1";
//synthesis attribute PREG of MADD4 is "1";
//synthesis attribute OPMODEREG of MADD4 is "0";
//synthesis attribute SUBTRACTREG of MADD4 is "0";
//synthesis attribute CARRYINSELREG of MADD4 is "0";
//synthesis attribute B_INPUT of MADD4 is "CASCADE";

//synthesis translate_off 
defparam MADD4.AREG = 2'b01;
defparam MADD4.BREG = 2'b10;
defparam MADD4.CARRYINREG = 1'b0;
defparam MADD4.MREG = 1'b1;
defparam MADD4.PREG = 1'b1;
defparam MADD4.OPMODEREG = 1'b0;
defparam MADD4.SUBTRACTREG = 1'b0;
defparam MADD4.CARRYINSELREG = 1'b0;
defparam MADD4.B_INPUT = "CASCADE";
//synthesis translate_on


//          
// Tap 5 Instantiation block 6
//

DSP48 MADD5
  (
    .A(H5),             // Input A to Multiplier
    .B(18'b0),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(BCOUT_4),          //
    .PCIN(PCOUT_4),          //
    .OPMODE(7'b0010101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
	 .CECINSUB(CE),       //
    .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(BCOUT_5),       //
    .P(),                  //
    .PCOUT(PCOUT_5)        //
  );

//synthesis attribute AREG of MADD5 is "1";
//synthesis attribute BREG of MADD5 is "2";
//synthesis attribute CARRYINREG of MADD5 is "0";
//synthesis attribute MREG of MADD5 is "1";
//synthesis attribute PREG of MADD5 is "1";
//synthesis attribute OPMODEREG of MADD5 is "0";
//synthesis attribute SUBTRACTREG of MADD5 is "0";
//synthesis attribute CARRYINSELREG of MADD5 is "0";
//synthesis attribute B_INPUT of MADD5 is "CASCADE";

//synthesis translate_off 
defparam MADD5.AREG = 2'b01;
defparam MADD5.BREG = 2'b10;
defparam MADD5.CARRYINREG = 1'b0;
defparam MADD5.MREG = 1'b1;
defparam MADD5.PREG = 1'b1;
defparam MADD5.OPMODEREG = 1'b0;
defparam MADD5.SUBTRACTREG = 1'b0;
defparam MADD5.CARRYINSELREG = 1'b0;
defparam MADD5.B_INPUT = "CASCADE";
//synthesis translate_on


//          
// Tap 6 Instantiation block 7
//

DSP48 MADD6
  (
    .A(H6),             // Input A to Multiplier
    .B(18'b0),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(BCOUT_5),          //
    .PCIN(PCOUT_5),          //
    .OPMODE(7'b0010101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
	 .CECINSUB(CE),       //
    .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(BCOUT_6),       //
    .P(),                  //
    .PCOUT(PCOUT_6)        //
  );

//synthesis attribute AREG of MADD6 is "1";
//synthesis attribute BREG of MADD6 is "2";
//synthesis attribute CARRYINREG of MADD6 is "0";
//synthesis attribute MREG of MADD6 is "1";
//synthesis attribute PREG of MADD6 is "1";
//synthesis attribute OPMODEREG of MADD6 is "0";
//synthesis attribute SUBTRACTREG of MADD6 is "0";
//synthesis attribute CARRYINSELREG of MADD6 is "0";
//synthesis attribute B_INPUT of MADD6 is "CASCADE";

//synthesis translate_off 
defparam MADD6.AREG = 2'b01;
defparam MADD6.BREG = 2'b10;
defparam MADD6.CARRYINREG = 1'b0;
defparam MADD6.MREG = 1'b1;
defparam MADD6.PREG = 1'b1;
defparam MADD6.OPMODEREG = 1'b0;
defparam MADD6.SUBTRACTREG = 1'b0;
defparam MADD6.CARRYINSELREG = 1'b0;
defparam MADD6.B_INPUT = "CASCADE";
//synthesis translate_on


//          
// Tap 7 Instantiation block 8
//

DSP48 MADD7
  (
    .A(H7),             // Input A to Multiplier
    .B(18'b0),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(BCOUT_6),          //
    .PCIN(PCOUT_6),          //
    .OPMODE(7'b0010101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
	 .CECINSUB(CE),       //
    .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(BCOUT_7),       //
    .P(),                  //
    .PCOUT(PCOUT_7)        //
  );

//synthesis attribute AREG of MADD7 is "1";
//synthesis attribute BREG of MADD7 is "2";
//synthesis attribute CARRYINREG of MADD7 is "0";
//synthesis attribute MREG of MADD7 is "1";
//synthesis attribute PREG of MADD7 is "1";
//synthesis attribute OPMODEREG of MADD7 is "0";
//synthesis attribute SUBTRACTREG of MADD7 is "0";
//synthesis attribute CARRYINSELREG of MADD7 is "0";
//synthesis attribute B_INPUT of MADD7 is "CASCADE";

//synthesis translate_off 
defparam MADD7.AREG = 2'b01;
defparam MADD7.BREG = 2'b10;
defparam MADD7.CARRYINREG = 1'b0;
defparam MADD7.MREG = 1'b1;
defparam MADD7.PREG = 1'b1;
defparam MADD7.OPMODEREG = 1'b0;
defparam MADD7.SUBTRACTREG = 1'b0;
defparam MADD7.CARRYINSELREG = 1'b0;
defparam MADD7.B_INPUT = "CASCADE";
//synthesis translate_on


//          
// Tap 8 Instantiation block 9
//

DSP48 MADD8
  (
    .A(H8),             // Input A to Multiplier
    .B(18'b0),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(BCOUT_7),          //
    .PCIN(PCOUT_7),          //
    .OPMODE(7'b0010101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
	 .CECINSUB(CE),       //
    .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(BCOUT_8),       //
    .P(),                  //
    .PCOUT(PCOUT_8)        //
  );

//synthesis attribute AREG of MADD8 is "1";
//synthesis attribute BREG of MADD8 is "2";
//synthesis attribute CARRYINREG of MADD8 is "0";
//synthesis attribute MREG of MADD8 is "1";
//synthesis attribute PREG of MADD8 is "1";
//synthesis attribute OPMODEREG of MADD8 is "0";
//synthesis attribute SUBTRACTREG of MADD8 is "0";
//synthesis attribute CARRYINSELREG of MADD8 is "0";
//synthesis attribute B_INPUT of MADD8 is "CASCADE";

//synthesis translate_off 
defparam MADD8.AREG = 2'b01;
defparam MADD8.BREG = 2'b10;
defparam MADD8.CARRYINREG = 1'b0;
defparam MADD8.MREG = 1'b1;
defparam MADD8.PREG = 1'b1;
defparam MADD8.OPMODEREG = 1'b0;
defparam MADD8.SUBTRACTREG = 1'b0;
defparam MADD8.CARRYINSELREG = 1'b0;
defparam MADD8.B_INPUT = "CASCADE";
//synthesis translate_on

//          
// Tap 9 Instantiation block 10
//

DSP48 MADD9
  (
    .A(H9),             // Input A to Multiplier
    .B(18'b0),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(BCOUT_8),          //
    .PCIN(PCOUT_8),          //
    .OPMODE(7'b0010101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
	 .CECINSUB(CE),       //
    .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(BCOUT_9),       //
    .P(),                  //
    .PCOUT(PCOUT_9)        //
  );


//synthesis attribute AREG of MADD9 is "1";
//synthesis attribute BREG of MADD9 is "2";
//synthesis attribute CARRYINREG of MADD9 is "0";
//synthesis attribute MREG of MADD9 is "1";
//synthesis attribute PREG of MADD9 is "1";
//synthesis attribute OPMODEREG of MADD9 is "0";
//synthesis attribute SUBTRACTREG of MADD9 is "0";
//synthesis attribute CARRYINSELREG of MADD9 is "0";
//synthesis attribute B_INPUT of MADD9 is "CASCADE";

//synthesis translate_off 
defparam MADD9.AREG = 2'b01;
defparam MADD9.BREG = 2'b10;
defparam MADD9.CARRYINREG = 1'b0;
defparam MADD9.MREG = 1'b1;
defparam MADD9.PREG = 1'b1;
defparam MADD9.OPMODEREG = 1'b0;
defparam MADD9.SUBTRACTREG = 1'b0;
defparam MADD9.CARRYINSELREG = 1'b0;
defparam MADD9.B_INPUT = "CASCADE";
//synthesis translate_on



DSP48 MADD10
  (
    .A(H10),             // Input A to Multiplier
    .B(18'b0),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(BCOUT_9),          //
    .PCIN(PCOUT_9),          //
    .OPMODE(7'b0010101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
	 .CECINSUB(CE),       //
    .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(BCOUT_10),       //
    .P(),                  //
    .PCOUT(PCOUT_10)        //
  );

//synthesis attribute AREG of MADD10 is "1";
//synthesis attribute BREG of MADD10 is "2";
//synthesis attribute CARRYINREG of MADD10 is "0";
//synthesis attribute MREG of MADD10 is "1";
//synthesis attribute PREG of MADD10 is "1";
//synthesis attribute OPMODEREG of MADD10 is "0";
//synthesis attribute SUBTRACTREG of MADD10 is "0";
//synthesis attribute CARRYINSELREG of MADD10 is "0";
//synthesis attribute B_INPUT of MADD10 is "CASCADE";

//synthesis translate_off 
defparam MADD10.AREG = 2'b01;
defparam MADD10.BREG = 2'b10;
defparam MADD10.CARRYINREG = 1'b0;
defparam MADD10.MREG = 1'b1;
defparam MADD10.PREG = 1'b1;
defparam MADD10.OPMODEREG = 1'b0;
defparam MADD10.SUBTRACTREG = 1'b0;
defparam MADD10.CARRYINSELREG = 1'b0;
defparam MADD10.B_INPUT = "CASCADE";
//synthesis translate_on


//          
// Tap 11 Instantiation block 12
//

DSP48 MADD11
  (
    .A(H11),             // Input A to Multiplier
    .B(18'b0),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(BCOUT_10),          //
    .PCIN(PCOUT_10),          //
    .OPMODE(7'b0010101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
	 .CECINSUB(CE),       //
    .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(BCOUT_11),       //
    .P(),                  //
    .PCOUT(PCOUT_11)        //
  );

//synthesis attribute AREG of MADD11 is "1";
//synthesis attribute BREG of MADD11 is "2";
//synthesis attribute CARRYINREG of MADD11 is "0";
//synthesis attribute MREG of MADD11 is "1";
//synthesis attribute PREG of MADD11 is "1";
//synthesis attribute OPMODEREG of MADD11 is "0";
//synthesis attribute SUBTRACTREG of MADD11 is "0";
//synthesis attribute CARRYINSELREG of MADD11 is "0";
//synthesis attribute B_INPUT of MADD11 is "CASCADE";

//synthesis translate_off 
defparam MADD11.AREG = 2'b01;
defparam MADD11.BREG = 2'b10;
defparam MADD11.CARRYINREG = 1'b0;
defparam MADD11.MREG = 1'b1;
defparam MADD11.PREG = 1'b1;
defparam MADD11.OPMODEREG = 1'b0;
defparam MADD11.SUBTRACTREG = 1'b0;
defparam MADD11.CARRYINSELREG = 1'b0;
defparam MADD11.B_INPUT = "CASCADE";
//synthesis translate_on


//          
// Tap 12 Instantiation block 13
//

DSP48 MADD12
  (
    .A(H12),             // Input A to Multiplier
    .B(18'b0),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(BCOUT_11),          //
    .PCIN(PCOUT_11),          //
    .OPMODE(7'b0010101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
	 .CECINSUB(CE),       //
    .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(BCOUT_12),       //
    .P(),                  //
    .PCOUT(PCOUT_12)        //
  );

//synthesis attribute AREG of MADD12 is "1";
//synthesis attribute BREG of MADD12 is "2";
//synthesis attribute CARRYINREG of MADD12 is "0";
//synthesis attribute MREG of MADD12 is "1";
//synthesis attribute PREG of MADD12 is "1";
//synthesis attribute OPMODEREG of MADD12 is "0";
//synthesis attribute SUBTRACTREG of MADD12 is "0";
//synthesis attribute CARRYINSELREG of MADD12 is "0";
//synthesis attribute B_INPUT of MADD12 is "CASCADE";

//synthesis translate_off 
defparam MADD12.AREG = 2'b01;
defparam MADD12.BREG = 2'b10;
defparam MADD12.CARRYINREG = 1'b0;
defparam MADD12.MREG = 1'b1;
defparam MADD12.PREG = 1'b1;
defparam MADD12.OPMODEREG = 1'b0;
defparam MADD12.SUBTRACTREG = 1'b0;
defparam MADD12.CARRYINSELREG = 1'b0;
defparam MADD12.B_INPUT = "CASCADE";
//synthesis translate_on


//          
// Tap 13 Instantiation block 14
//

DSP48 MADD13
  (
    .A(H13),             // Input A to Multiplier
    .B(18'b0),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(BCOUT_12),          //
    .PCIN(PCOUT_12),          //
    .OPMODE(7'b0010101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
	 .CECINSUB(CE),       //
    .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(BCOUT_13),       //
    .P(),                  //
    .PCOUT(PCOUT_13)        //
  );

//synthesis attribute AREG of MADD13 is "1";
//synthesis attribute BREG of MADD13 is "2";
//synthesis attribute CARRYINREG of MADD13 is "0";
//synthesis attribute MREG of MADD13 is "1";
//synthesis attribute PREG of MADD13 is "1";
//synthesis attribute OPMODEREG of MADD13 is "0";
//synthesis attribute SUBTRACTREG of MADD13 is "0";
//synthesis attribute CARRYINSELREG of MADD13 is "0";
//synthesis attribute B_INPUT of MADD13 is "CASCADE";

//synthesis translate_off 
defparam MADD13.AREG = 2'b01;
defparam MADD13.BREG = 2'b10;
defparam MADD13.CARRYINREG = 1'b0;
defparam MADD13.MREG = 1'b1;
defparam MADD13.PREG = 1'b1;
defparam MADD13.OPMODEREG = 1'b0;
defparam MADD13.SUBTRACTREG = 1'b0;
defparam MADD13.CARRYINSELREG = 1'b0;
defparam MADD13.B_INPUT = "CASCADE";
//synthesis translate_on

//          
// Tap 14 Instantiation block 15
//

DSP48 MADD14
  (
    .A(H14),             // Input A to Multiplier
    .B(18'b0),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(BCOUT_13),          //
    .PCIN(PCOUT_13),          //
    .OPMODE(7'b0010101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
    .CECINSUB(CE),       //
	 .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(BCOUT_14),       //
    .P(),                  //
    .PCOUT(PCOUT_14)        //
  );

//synthesis attribute AREG of MADD14 is "1";
//synthesis attribute BREG of MADD14 is "2";
//synthesis attribute CARRYINREG of MADD14 is "0";
//synthesis attribute MREG of MADD14 is "1";
//synthesis attribute PREG of MADD14 is "1";
//synthesis attribute OPMODEREG of MADD14 is "0";
//synthesis attribute SUBTRACTREG of MADD14 is "0";
//synthesis attribute CARRYINSELREG of MADD14 is "0";
//synthesis attribute B_INPUT of MADD14 is "CASCADE";

//synthesis translate_off 
defparam MADD14.AREG = 2'b01;
defparam MADD14.BREG = 2'b10;
defparam MADD14.CARRYINREG = 1'b0;
defparam MADD14.MREG = 1'b1;
defparam MADD14.PREG = 1'b1;
defparam MADD14.OPMODEREG = 1'b0;
defparam MADD14.SUBTRACTREG = 1'b0;
defparam MADD14.CARRYINSELREG = 1'b0;
defparam MADD14.B_INPUT = "CASCADE";
//synthesis translate_on


//          
// Tap 15 Instantiation block 16
//

DSP48 MADD15
  (
    .A(H15),             // Input A to Multiplier
    .B(18'b0),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(BCOUT_14),          //
    .PCIN(PCOUT_14),          //
    .OPMODE(7'b0010101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
    .CECINSUB(CE),       //
	 .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(BCOUT_15),       //
    .P(),                  //
    .PCOUT(PCOUT_15)        //
  );

//synthesis attribute AREG of MADD15 is "1";
//synthesis attribute BREG of MADD15 is "2";
//synthesis attribute CARRYINREG of MADD15 is "0";
//synthesis attribute MREG of MADD15 is "1";
//synthesis attribute PREG of MADD15 is "1";
//synthesis attribute OPMODEREG of MADD15 is "0";
//synthesis attribute SUBTRACTREG of MADD15 is "0";
//synthesis attribute CARRYINSELREG of MADD15 is "0";
//synthesis attribute B_INPUT of MADD15 is "CASCADE";

//synthesis translate_off 
defparam MADD15.AREG = 2'b01;
defparam MADD15.BREG = 2'b10;
defparam MADD15.CARRYINREG = 1'b0;
defparam MADD15.MREG = 1'b1;
defparam MADD15.PREG = 1'b1;
defparam MADD15.OPMODEREG = 1'b0;
defparam MADD15.SUBTRACTREG = 1'b0;
defparam MADD15.CARRYINSELREG = 1'b0;
defparam MADD15.B_INPUT = "CASCADE";
//synthesis translate_on


//          
// Tap 16 Instantiation block 17
//

DSP48 MADD16
  (
    .A(H16),             // Input A to Multiplier
    .B(18'b0),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(BCOUT_15),          //
    .PCIN(PCOUT_15),          //
    .OPMODE(7'b0010101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
    .CECINSUB(CE),       //
	 .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(BCOUT_16),       //
    .P(),                  //
    .PCOUT(PCOUT_16)        //
  );

//synthesis attribute AREG of MADD16 is "1";
//synthesis attribute BREG of MADD16 is "2";
//synthesis attribute CARRYINREG of MADD16 is "0";
//synthesis attribute MREG of MADD16 is "1";
//synthesis attribute PREG of MADD16 is "1";
//synthesis attribute OPMODEREG of MADD16 is "0";
//synthesis attribute SUBTRACTREG of MADD16 is "0";
//synthesis attribute CARRYINSELREG of MADD16 is "0";
//synthesis attribute B_INPUT of MADD16 is "CASCADE";

//synthesis translate_off 
defparam MADD16.AREG = 2'b01;
defparam MADD16.BREG = 2'b10;
defparam MADD16.CARRYINREG = 1'b0;
defparam MADD16.MREG = 1'b1;
defparam MADD16.PREG = 1'b1;
defparam MADD16.OPMODEREG = 1'b0;
defparam MADD16.SUBTRACTREG = 1'b0;
defparam MADD16.CARRYINSELREG = 1'b0;
defparam MADD16.B_INPUT = "CASCADE";
//synthesis translate_on


//          
// Tap 17 Instantiation block 18
//

DSP48 MADD17
  (
    .A(H17),             // Input A to Multiplier
    .B(18'b0),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(BCOUT_16),          //
    .PCIN(PCOUT_16),          //
    .OPMODE(7'b0010101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
    .CECINSUB(CE),       //
	 .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(BCOUT_17),       //
    .P(),                  //
    .PCOUT(PCOUT_17)        //
  );

//synthesis attribute AREG of MADD17 is "1";
//synthesis attribute BREG of MADD17 is "2";
//synthesis attribute CARRYINREG of MADD17 is "0";
//synthesis attribute MREG of MADD17 is "1";
//synthesis attribute PREG of MADD17 is "1";
//synthesis attribute OPMODEREG of MADD17 is "0";
//synthesis attribute SUBTRACTREG of MADD17 is "0";
//synthesis attribute CARRYINSELREG of MADD17 is "0";
//synthesis attribute B_INPUT of MADD17 is "CASCADE";

//synthesis translate_off 
defparam MADD17.AREG = 2'b01;
defparam MADD17.BREG = 2'b10;
defparam MADD17.CARRYINREG = 1'b0;
defparam MADD17.MREG = 1'b1;
defparam MADD17.PREG = 1'b1;
defparam MADD17.OPMODEREG = 1'b0;
defparam MADD17.SUBTRACTREG = 1'b0;
defparam MADD17.CARRYINSELREG = 1'b0;
defparam MADD17.B_INPUT = "CASCADE";
//synthesis translate_on

//          
// Tap 18 Instantiation block 19
//

DSP48 MADD18
  (
    .A(H18),             // Input A to Multiplier
    .B(18'b0),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(BCOUT_17),          //
    .PCIN(PCOUT_17),          //
    .OPMODE(7'b0010101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CECINSUB(CE),       //
	 .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
    .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(BCOUT_18),       //
    .P(),                  //
    .PCOUT(PCOUT_18)        //
  );

//synthesis attribute AREG of MADD18 is "1";
//synthesis attribute BREG of MADD18 is "2";
//synthesis attribute CARRYINREG of MADD18 is "0";
//synthesis attribute MREG of MADD18 is "1";
//synthesis attribute PREG of MADD18 is "1";
//synthesis attribute OPMODEREG of MADD18 is "0";
//synthesis attribute SUBTRACTREG of MADD18 is "0";
//synthesis attribute CARRYINSELREG of MADD18 is "0";
//synthesis attribute B_INPUT of MADD18 is "CASCADE";

//synthesis translate_off 
defparam MADD18.AREG = 2'b01;
defparam MADD18.BREG = 2'b10;
defparam MADD18.CARRYINREG = 1'b0;
defparam MADD18.MREG = 1'b1;
defparam MADD18.PREG = 1'b1;
defparam MADD18.OPMODEREG = 1'b0;
defparam MADD18.SUBTRACTREG = 1'b0;
defparam MADD18.CARRYINSELREG = 1'b0;
defparam MADD18.B_INPUT = "CASCADE";
//synthesis translate_on 	


//          
// Tap 19 Instantiation block 20
//

DSP48 MADD19
  (
    .A(H19),             // Input A to Multiplier
    .B(18'b0),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(BCOUT_18),          //
    .PCIN(PCOUT_18),          //
    .OPMODE(7'b0010101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CECINSUB(CE),       //
	 .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
    .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(BCOUT_19),       //
    .P(),                  //
    .PCOUT(PCOUT_19)        //
  );

//synthesis attribute AREG of MADD19 is "1";
//synthesis attribute BREG of MADD19 is "2";
//synthesis attribute CARRYINREG of MADD19 is "0";
//synthesis attribute MREG of MADD19 is "1";
//synthesis attribute PREG of MADD19 is "1";
//synthesis attribute OPMODEREG of MADD19 is "0";
//synthesis attribute SUBTRACTREG of MADD19 is "0";
//synthesis attribute CARRYINSELREG of MADD19 is "0";
//synthesis attribute B_INPUT of MADD19 is "CASCADE";

//synthesis translate_off 
defparam MADD19.AREG = 2'b01;
defparam MADD19.BREG = 2'b10;
defparam MADD19.CARRYINREG = 1'b0;
defparam MADD19.MREG = 1'b1;
defparam MADD19.PREG = 1'b1;
defparam MADD19.OPMODEREG = 1'b0;
defparam MADD19.SUBTRACTREG = 1'b0;
defparam MADD19.CARRYINSELREG = 1'b0;
defparam MADD19.B_INPUT = "CASCADE";
//synthesis translate_on 


//          
// Tap 20 Instantiation block 21
//

DSP48 MADD20
  (
    .A(H20),             // Input A to Multiplier
    .B(18'b0),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(BCOUT_19),          //
    .PCIN(PCOUT_19),          //
    .OPMODE(7'b0010101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
    .CECINSUB(CE),       //
	 .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(BCOUT_20),       //
    .P(),                  //
    .PCOUT(PCOUT_20)        //
  );

//synthesis attribute AREG of MADD20 is "1";
//synthesis attribute BREG of MADD20 is "2";
//synthesis attribute CARRYINREG of MADD20 is "0";
//synthesis attribute MREG of MADD20 is "1";
//synthesis attribute PREG of MADD20 is "1";
//synthesis attribute OPMODEREG of MADD20 is "0";
//synthesis attribute SUBTRACTREG of MADD20 is "0";
//synthesis attribute CARRYINSELREG of MADD20 is "0";
//synthesis attribute B_INPUT of MADD20 is "CASCADE";

//synthesis translate_off 
defparam MADD20.AREG = 2'b01;
defparam MADD20.BREG = 2'b10;
defparam MADD20.CARRYINREG = 1'b0;
defparam MADD20.MREG = 1'b1;
defparam MADD20.PREG = 1'b1;
defparam MADD20.OPMODEREG = 1'b0;
defparam MADD20.SUBTRACTREG = 1'b0;
defparam MADD20.CARRYINSELREG = 1'b0;
defparam MADD20.B_INPUT = "CASCADE";
//synthesis translate_on 

//          
// Tap 21 Instantiation block 22
//

DSP48 MADD21
  (
    .A(H21),             // Input A to Multiplier
    .B(18'b0),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(BCOUT_20),          //
    .PCIN(PCOUT_20),          //
    .OPMODE(7'b0010101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
    .CECINSUB(CE),       //
	 .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(BCOUT_21),       //
    .P(),                  //
    .PCOUT(PCOUT_21)        //
  );

//synthesis attribute AREG of MADD21 is "1";
//synthesis attribute BREG of MADD21 is "2";
//synthesis attribute CARRYINREG of MADD21 is "0";
//synthesis attribute MREG of MADD21 is "1";
//synthesis attribute PREG of MADD21 is "1";
//synthesis attribute OPMODEREG of MADD21 is "0";
//synthesis attribute SUBTRACTREG of MADD21 is "0";
//synthesis attribute CARRYINSELREG of MADD21 is "0";
//synthesis attribute B_INPUT of MADD21 is "CASCADE";

//synthesis translate_off 
defparam MADD21.AREG = 2'b01;
defparam MADD21.BREG = 2'b10;
defparam MADD21.CARRYINREG = 1'b0;
defparam MADD21.MREG = 1'b1;
defparam MADD21.PREG = 1'b1;
defparam MADD21.OPMODEREG = 1'b0;
defparam MADD21.SUBTRACTREG = 1'b0;
defparam MADD21.CARRYINSELREG = 1'b0;
defparam MADD21.B_INPUT = "CASCADE";
//synthesis translate_on 


//          
// Tap 22 Instantiation block 23
//

DSP48 MADD22
  (
    .A(H22),             // Input A to Multiplier
    .B(18'b0),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(BCOUT_21),          //
    .PCIN(PCOUT_21),          //
    .OPMODE(7'b0010101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
    .CECINSUB(CE),       //
	 .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(),       //
    .P(DOUT),                  //
    .PCOUT()        //
  );

//synthesis attribute AREG of MADD22 is "1";
//synthesis attribute BREG of MADD22 is "2";
//synthesis attribute CARRYINREG of MADD22 is "0";
//synthesis attribute MREG of MADD22 is "1";
//synthesis attribute PREG of MADD22 is "1";
//synthesis attribute OPMODEREG of MADD22 is "0";
//synthesis attribute SUBTRACTREG of MADD22 is "0";
//synthesis attribute CARRYINSELREG of MADD22 is "0";
//synthesis attribute B_INPUT of MADD22 is "CASCADE";

//synthesis translate_off 
defparam MADD22.AREG = 2'b01;
defparam MADD22.BREG = 2'b10;
defparam MADD22.CARRYINREG = 1'b0;
defparam MADD22.MREG = 1'b1;
defparam MADD22.PREG = 1'b1;
defparam MADD22.OPMODEREG = 1'b0;
defparam MADD22.SUBTRACTREG = 1'b0;
defparam MADD22.CARRYINSELREG = 1'b0;
defparam MADD22.B_INPUT = "CASCADE";
//synthesis translate_on 

//
////////////////////////////////////////////////////////////////////////////////////

endmodule

module DSP48 (BCOUT, P, PCOUT, A, B, C, OPMODE, SUBTRACT, CARRYIN, CECINSUB, CARRYINSEL,
              BCIN, PCIN, CLK, CEA, CEB, CEC, CEP, CEM, CECTRL, CECARRYIN,
			  RSTA, RSTB, RSTC, RSTP, RSTM, RSTCTRL, RSTCARRYIN) /* synthesis syn_black_box */;
		output [47:0] P, PCOUT;
		output [17:0] BCOUT;

		input [17:0] A, B, BCIN;
		input [47:0] PCIN,C;

		input [6:0] OPMODE;
		input [1:0] CARRYINSEL;
		input CLK, CEA, CEB, CEC, CEP, CEM, CECTRL, SUBTRACT, CARRYIN, CECARRYIN, CECINSUB;
		input RSTA, RSTB, RSTC, RSTP, RSTM, RSTCTRL, RSTCARRYIN;

endmodule

