--  Example of a Systolic Parallel 18 x 18 N Tap FIR Filter
-------------------------------------------------------------------------------- 
-- Copyright (c) 2004 Xilinx, Inc. 
-- All Rights Reserved 
-------------------------------------------------------------------------------- 
--   ____  ____ 
--  /   /\/   / 
-- /___/  \  /   Vendor: Xilinx 
-- \   \   \/    Author: Niall Battson, Xilinx, Inc.
--  \   \        Filename: design_top.vhd
--  /   /        Date Last Modified:  Sept 2, 2004 
-- /___/   /\    Date Created: Sept 2, 2004 
-- \   \  /  \ 
--  \___\/\___\ 
-- 
--
-- Revision History: 
-------------------------------------------------------------------------------- 
--   
--   XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" 
--   AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND 
--   SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, 
--   OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, 
--   APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION 
--   THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT, 
--   AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE 
--   FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY 
--   WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE 
--   IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR 
--   REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF 
--   INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
--   FOR A PARTICULAR PURPOSE. 
--
-------------------------------------------------------------------------------- 


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity design_top is
    Port ( xn : in std_logic_vector(17 downto 0);
           yn : out std_logic_vector(47 downto 0);
           clk : in std_logic;
           ce : in std_logic;
           rst : in std_logic);
end design_top;

architecture Behavioral of design_top is

------------------------------------------------------------------------
--
-- Component Declarations:
--
------------------------------------------------------------------------

component MADD
    generic (
           opmode: std_logic_vector(6 downto 0);
			  coef_value: integer := 0);

	 Port ( din : in std_logic_vector(17 downto 0);
           pcin : in std_logic_vector(47 downto 0);
           clk : in std_logic;
           ce : in std_logic;
           rst : in std_logic;
			  bcout : out std_logic_vector(17 downto 0);
			  dout : out std_logic_vector(47 downto 0)
			  );
  end component;

  component MADD_FIRST
    generic (
           opmode: std_logic_vector(6 downto 0);
			  coef_value: integer := 0);

	 Port ( din : in std_logic_vector(17 downto 0);
           pcin : in std_logic_vector(47 downto 0);
           clk : in std_logic;
           ce : in std_logic;
           rst : in std_logic;
			  bcout : out std_logic_vector(17 downto 0);
			  dout : out std_logic_vector(47 downto 0)
			  );
  end component;

  component MADD_LAST
    generic (
           opmode: std_logic_vector(6 downto 0);
			  coef_value: integer := 0);

	 Port ( din : in std_logic_vector(17 downto 0);
           pcin : in std_logic_vector(47 downto 0);
           clk : in std_logic;
           ce : in std_logic;
           rst : in std_logic;
			  bcout : out std_logic_vector(17 downto 0);
			  dout : out std_logic_vector(47 downto 0)
			  );
  end component;

----------------------------------------------------------
--     Number of Coefficients					              --
----------------------------------------------------------
  constant n: integer := 23;
  type coef_array is array (0 to n - 1) of integer;
  
----------------------------------------------------------
--     Coefficient Values      				              --
----------------------------------------------------------  
  
  constant coefficient: coef_array := (
     -38, -74, -109, -109, -37, 140, 435, 827, 1262, 1663, 1945, 2047,
     1945, 1663, 1262, 827, 435, 140, -37, -109, -109, -74, -38
  );

---------------------------------------------------------------------------------
--     Setup all the connecting signals between MADD elements in one vector    --
---------------------------------------------------------------------------------
  
  signal pcin_temp : std_logic_vector(48 * n downto 0) := conv_std_logic_vector(0, 48 * n + 1);
  signal bcout_temp : std_logic_vector(48 * n downto 0) := conv_std_logic_vector(0, 48 * n + 1);


begin
  G0: for i in 0 to n - 1 generate

  G_first: if i = 0 generate
    M0: MADD_FIRST
      generic map (
		  opmode => "0000101",
        coef_value => coefficient(i)
      )
      port map (
        din => xn,
        pcin => "000000000000000000000000000000000000000000000000",
        clk => clk,
        ce => ce,
        rst => rst,
		  bcout => bcout_temp(17 downto 0),
        dout => pcin_temp(47 downto 0)
      );
  end generate;

  GX: if (i >= 1 and i < n - 1) generate
    M1: MADD
      generic map (
        opmode => "0010101",
		  coef_value => coefficient(i)
      )
      port map (
        din => bcout_temp(i * 17 + (i - 1) downto ((i - 1) * 18)), 
		  pcin => pcin_temp(i * 47 + (i - 1) downto ((i - 1) * 48)),
        clk => clk,
        ce => ce,
        rst => rst,
		  bcout => bcout_temp((i + 1) * 17 + i downto i * 18),
        dout => pcin_temp((i + 1) * 47 + i downto i * 48)
      );
  end generate;

  G_last: if i = n - 1 generate
    M2: MADD_LAST
      generic map (
        opmode => "0010101",
		  coef_value => coefficient(i)
      )
      port map (
        din => bcout_temp(i * 17 + (i - 1) downto ((i - 1) * 18)),
		  pcin => pcin_temp(i * 47 + (i - 1) downto ((i - 1) * 48)),
        clk => clk,
        ce => ce,
        rst => rst,
		  bcout => open,
        dout => yn
      );
    end generate;
  end generate;
end Behavioral;
