library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
library UNISIM;
use UNISIM.VComponents.all;

entity MADD is
    generic (
           opmode: std_logic_vector(6 downto 0);
			  coef_value: integer := 0);

	 Port ( din : in std_logic_vector(17 downto 0);
           pcin : in std_logic_vector(47 downto 0);
           clk : in std_logic;
           ce : in std_logic;
           rst : in std_logic;
			  bcout : out std_logic_vector(17 downto 0);
			  dout : out std_logic_vector(47 downto 0));
end MADD;

architecture Behavioral of MADD is

------------------------------------------------------------------------
--
-- Component Declarations:
--
------------------------------------------------------------------------

component DSP48
-- synthesis translate_off
  generic
  (
  AREG            : integer       := 1;
  B_INPUT         : string        := "CASCADE";
  BREG            : integer       := 2;
  CARRYINREG      : integer       := 1;
  CARRYINSELREG   : integer       := 1;
  CREG            : integer       := 1;
  LEGACY_MODE     : string        := "MULT18X18S";
  MREG            : integer       := 1;
  OPMODEREG       : integer       := 1;
  PREG            : integer       := 1;
  SUBTRACTREG     : integer       := 1;
  SIM_X_INPUT	   : string	       := "GENERATE_X_ONLY"
  );
-- synthesis translate_on

      PORT (
         BCOUT                   : OUT std_logic_vector(17 DOWNTO 0);
         P                       : OUT std_logic_vector(47 DOWNTO 0);
         PCOUT                   : OUT std_logic_vector(47 DOWNTO 0);
         A                : in  std_logic_vector(17 downto 0); -- input A to Multiplier 
         B                : in  std_logic_vector(17 downto 0); -- input B to Multiplier 
         C                : in  std_logic_vector(47 downto 0); -- input C to Adder 
         OPMODE                  : IN  std_logic_vector(6 DOWNTO 0);
         SUBTRACT                : IN  std_logic;
         CARRYIN                 : IN  std_logic;
         CARRYINSEL              : IN  std_logic_vector(1 DOWNTO 0);
         BCIN                    : IN  std_logic_vector(17 DOWNTO 0);
         PCIN                    : IN  std_logic_vector(47 DOWNTO 0);
         CLK                     : IN  std_logic;
         CEA                     : IN  std_logic;
         CEB                     : IN  std_logic;
         CEC                     : IN  std_logic;
         CEP                     : IN  std_logic;
         CEM                     : IN  std_logic;
         CECTRL                  : IN  std_logic;
         CECARRYIN               : IN  std_logic;
         CECINSUB						: IN  std_logic;
			RSTA                    : IN  std_logic;
         RSTB                    : IN  std_logic;
         RSTC                    : IN  std_logic;
         RSTP                    : IN  std_logic;
         RSTM                    : IN  std_logic;
         RSTCTRL                 : IN  std_logic;
         RSTCARRYIN              : IN  std_logic);
   END COMPONENT;

   signal coef: std_logic_vector(17 downto 0);



begin

coef <= conv_std_logic_vector(coef_value, 18);



   MADD1 : DSP48 
      PORT MAP (
         A => coef,
         B => "000000000000000000",
         C => "000000000000000000000000000000000000000000000000",
         BCIN => din,
         PCIN => pcin,
         OPMODE => opmode,
         SUBTRACT => '0',
         CARRYIN => '0',
         CARRYINSEL => "00",
         CLK => CLK,
         CEA => CE,
         CEB => CE,
         CEC => CE,
         CEP => CE,
         CEM => CE,
         CECTRL => CE,
         CECARRYIN => CE,
         CECINSUB => CE,
			RSTA => RST,
         RSTB => RST,
         RSTC => RST,
         RSTP => RST,
         RSTM => RST,
         RSTCTRL => RST,
         RSTCARRYIN => RST,
         BCOUT => bcout,
         P => open,
         PCOUT => dout);

end Behavioral;
